@@ -32,36 +32,39 @@ static void eof_callback(LinphonePlayer *player, void *user_data) {
 	*eof = TRUE;
 }
 
-static void play_file(const char *filename, bool_t unsupported_format, const char *audio_mime, const char *video_mime) {
+static void play_file(const char *filename, bool_t supported_format, const char *audio_mime, const char *video_mime) {
 	LinphoneCoreManager *lc_manager;
 	LinphonePlayer *player;
 	int res, time = 0;
 	bool_t eof = FALSE;
+	bool_t audio_codec_supported;
+	bool_t video_codec_supported;
 
 	lc_manager = linphone_core_manager_new("marie_rc");
-	CU_ASSERT_PTR_NOT_NULL(lc_manager);
+	BC_ASSERT_PTR_NOT_NULL(lc_manager);
 	if(lc_manager == NULL) return;
+	
+	audio_codec_supported = (audio_mime && ms_factory_get_decoder(ms_factory_get_fallback(), audio_mime));
+	video_codec_supported = (video_mime && ms_factory_get_decoder(ms_factory_get_fallback(), video_mime));
 
 	player = linphone_core_create_local_player(lc_manager->lc, ms_snd_card_manager_get_default_card(ms_snd_card_manager_get()), video_stream_get_default_video_renderer(), 0);
-	CU_ASSERT_PTR_NOT_NULL(player);
+	BC_ASSERT_PTR_NOT_NULL(player);
 	if(player == NULL) goto fail;
 
 	res = linphone_player_open(player, filename, eof_callback, &eof);
-	if(unsupported_format
-			|| (audio_mime == NULL && video_mime == NULL)
-			|| (video_mime == NULL && audio_mime && !ms_filter_codec_supported(audio_mime))
-			|| (audio_mime == NULL && video_mime && !ms_filter_codec_supported(video_mime))) {
-		CU_ASSERT_EQUAL(res, -1);
+	if(supported_format && (audio_codec_supported || video_codec_supported)) {
+		BC_ASSERT_EQUAL(res, 0, int, "%d");
 	} else {
-		CU_ASSERT_EQUAL(res, 0);
+		BC_ASSERT_EQUAL(res, -1, int, "%d");
 	}
+	
 	if(res == -1) goto fail;
 
 	res = linphone_player_start(player);
-	CU_ASSERT_EQUAL(res, 0);
+	BC_ASSERT_EQUAL(res, 0, int, "%d");
 	if(res == -1) goto fail;
 
-	CU_ASSERT_TRUE(wait_for_eof(&eof, &time, 100, 13000));
+	BC_ASSERT_TRUE(wait_for_eof(&eof, &time, 100, (int)(linphone_player_get_duration(player) * 1.05)));
 
 	linphone_player_close(player);
 
@@ -70,22 +73,35 @@ static void play_file(const char *filename, bool_t unsupported_format, const cha
 	if(lc_manager) linphone_core_manager_destroy(lc_manager);
 }
 
-static void playing_test(void) {
-	char *filename = ms_strdup_printf("%s/sounds/hello_opus_h264.mkv", bc_tester_read_dir_prefix);
+static void sintel_trailer_opus_h264_test(void) {
+	char *filename = bc_tester_res("sounds/sintel_trailer_opus_h264.mkv");
 	const char *audio_mime = "opus";
-	const char *video_mime = "h264";
-	play_file(filename, !linphone_local_player_matroska_supported(), audio_mime, video_mime);
+	const char *video_mime = "H264";
+	play_file(filename, linphone_local_player_matroska_supported(), audio_mime, video_mime);
+	ms_free(filename);
+}
+
+static void sintel_trailer_pcmu_h264_test(void) {
+	char *filename = bc_tester_res("sounds/sintel_trailer_pcmu_h264.mkv");
+	const char *audio_mime = "pcmu";
+	const char *video_mime = "H264";
+	play_file(filename, linphone_local_player_matroska_supported(), audio_mime, video_mime);
+	ms_free(filename);
+}
+
+static void sintel_trailer_opus_vp8_test(void) {
+	char *filename = bc_tester_res("sounds/sintel_trailer_opus_vp8.mkv");
+	const char *audio_mime = "opus";
+	const char *video_mime = "VP8";
+	play_file(filename, linphone_local_player_matroska_supported(), audio_mime, video_mime);
 	ms_free(filename);
 }
 
 test_t player_tests[] = {
-	{	"Local MKV file"	,	playing_test	}
+	{	"Sintel trailer opus/h264"	,	sintel_trailer_opus_h264_test	},
+	{	"Sintel trailer pcmu/h264"	,	sintel_trailer_pcmu_h264_test	},
+	{	"Sintel trailer opus/VP8"	,	sintel_trailer_opus_vp8_test	}
 };
 
-test_suite_t player_test_suite = {
-	"Player",
-	NULL,
-	NULL,
-	sizeof(player_tests) / sizeof(test_t),
-	player_tests
-};
+test_suite_t player_test_suite = {"Player", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
+								  sizeof(player_tests) / sizeof(test_t), player_tests};
